//------------------------------------------------------------------------------
// JavaScript code for the Hifidelio Dashboard Widget "Fideline"
// © 2009 Bernd Löhr
// Partly borrowed from Apple's Dashboard Sample Code
//------------------------------------------------------------------------------

var gHFURL = "hifideliopro.local.";
var gStatusLink;
var gStateDoc;

//------------------------------------------------------------------------------
// Populate the Bonjour popup menu
//------------------------------------------------------------------------------
function updateDeviceList()
{
	var output = widget.system("./LocalChooser", null).outputString;
	
	if (typeof output == "undefined")
		return;

	var popupToChange = document.getElementById("popup");
	var devices = output.split("\n");
	var array = new Array();
	var iDevice = 0;

	// Check for Hifidelio(s)
	for (var i = 0; i < devices.length - 1; i++)
	{
		var line = devices[i].split("\t");
		var content = loadStatus('http://' + line[1] + '/cgi-bin/leonore.cgi?h=S');
		if (typeof content == "undefined")
			continue;
		if (content.length > 0)
		{
			array[iDevice] = devices[i];
			iDevice++;
		}
	}

	// Check if we need to change the number of menu items
	while (array.length > popupToChange.length)
	{
		var NeuerEintrag = new Option("additional", "", false, true);
		popupToChange.options[popupToChange.length] = NeuerEintrag;
	}

	while (array.length < popupToChange.length)
	{
		popupToChange.options[popupToChange.length - 1] = null;
	}

	// Early exit, no device found
	if (array.length == 0)
	{
		var NeuerEintrag = new Option("additional", "", false, true);
		popupToChange.options[0] = NeuerEintrag;
		popupToChange[0].value = "No Hifidelio";
		popupToChange[0].text = "No Hifidelio";
		return;
	}

	// Fill it up
	for (var i = 0; i < array.length; i++)
	{
		var line = array[i].split("\t");
		popupToChange[i].value = line[1];
		popupToChange[i].text = line[0];
		if (line[1] == gHFURL)
			popupToChange[i].selected = true;
		else
			popupToChange[i].selected = false;
	}

	selectDevice();
}

//------------------------------------------------------------------------------
// Called when a device was selected,
// save the selection for next startup
//------------------------------------------------------------------------------
function selectDevice()
{
	var popupValue = document.getElementById("popup");
	gHFURL = popupValue.value;
	widget.setPreferenceForKey(gHFURL,"hifidelioURL");		// save the new preference to disk
	gStatusLink = 'http://' + gHFURL + '/cgi-bin/leonore.cgi?h=S';
} 

//------------------------------------------------------------------------------
// This is the actual call to Leonore.
// Retrieve a complete status page
//------------------------------------------------------------------------------
function loadStatus(fileUrl)
{
	var req; 
	var fileContent; 

	try
	{
		req = new XMLHttpRequest(); 
		req.open("GET", fileUrl, false); 
		req.send(null);
		if (req.status != 200)
			return;
		fileContent = req.responseText;
	}
	catch (e)
	{
		fileContent = "";
	}
    return fileContent; 
} 

//------------------------------------------------------------------------------
// Set new attributes derived from Leonore
//------------------------------------------------------------------------------
function setAttrIfChanged (
	oldDoc,			// the old document
	newValue,		// the current value
	id,				// id of the element to change
	attr)			// attribute of the element containing the value to copy (if different)
{
	if (oldDoc && newValue)
	try	{
		var elmt1 = oldDoc.getElementById (id);

		if (elmt1) 
		{
		
			if ((attr == "class") && (elmt1.getAttribute (attr) == null))
				attr = "className";
			if ((attr == "width") && (elmt1.getAttribute ("class") == null))
			{
				if (newValue != oldDoc.getElementById(id).style.width);
					oldDoc.getElementById(id).style.width = newValue;
                return;
        	}
        	if (elmt1.getAttribute (attr) != newValue)
				elmt1.setAttribute (attr, newValue);
		}
	}
	catch (e)
	{
		//alert ("Exception caught: " + e);
	}
}

//------------------------------------------------------------------------------
// Modify local display with data derived from Leonore
//------------------------------------------------------------------------------
function setContentIfChanged  (
	oldDoc, 		// the old document
	newText,		// new string
	id)				// id of the element containing the data to copy (if different)
{
	if (oldDoc && newText)
	{
		try
		{
			var elmt1 = oldDoc.getElementById (id);

			if (elmt1)
			{
				if (! elmt1.firstChild)
				{
					var child = document.createTextNode (newText);
					elmt1.appendChild (child);
				}
				else
				{
					if (! elmt1.firstChild.data)
					{
						elmt1.firstChild.data = newText;
					}
					else
					{
						if (elmt1.firstChild.data != newText)
							elmt1.firstChild.data = newText;
					}
				}
			}
		}
		catch (e)
		{
	//		alert ("Exception caught: " + e);
		}
	}
}


//------------------------------------------------------------------------------
// This is called every 2 seconds
//------------------------------------------------------------------------------
function handlePlayerUpdate ()
{
	try
	{
		gStateDoc = loadStatus(gStatusLink);
		if (gStateDoc == "")
			return;

		// update only the known elements, and only if they changed:

		setAttrIfChanged (document, getCoverart(), 'coverart', 'class');
		setAttrIfChanged (document, getCoverPng(), 'coverart', 'src');
		setAttrIfChanged (document, getTimeline(), 'timelevelimg', 'width');
		setAttrIfChanged (document, getButtons(), 'playerkeylist', 'class');

		setContentIfChanged (document, getTime(), 'currtime');
		setContentIfChanged (document, getRemaining(), 'currremain');
		setContentIfChanged (document, getTrack(), 'currtrack');
		setContentIfChanged (document, getTitel(), 'currtitle');
		setContentIfChanged (document, getArtist(), 'currartist');
		setContentIfChanged (document, getAlbum(), 'curralbum');
		setContentIfChanged (document, getComposer(), 'currcomposer');
	}
	catch (e)
	{
		alert ("Exception caught: " + e);
	}
}

/***********************************/
// SAVING AND RETRIEVING PREFERENCES
/***********************************/

// setup() is run when the body loads.  It checks to see if there is a preference for this widget
// and if so, applies the preference to the widget.

function setup()
{
	if (window.widget)		// always check to make sure that we are running in Dashboard
	{
		// The preferences are retrieved:
		var worldString = widget.preferenceForKey("hifidelioURL");

		if (worldString && worldString.length > 0)  // if the retrieved preferences are not empty,
		{											// they are restored.
			//document.getElementById("urlField").value = worldString;
			gHFURL = worldString;
		}
		gStatusLink = 'http://' + gHFURL + '/cgi-bin/leonore.cgi?h=S&c=';
		setInterval('handlePlayerUpdate()', 2000);

		//	Populate the popup
		//updateDeviceList();
	}
}

// doButton controls the Hifidelio (back, stop, play/pause, forward)
function doButton(num)
{
	loadStatus('http://' + gHFURL + '/cgi-bin/leonore.cgi?d=K&h=P&o=' + num + '&c=');
	// Early update of button appearance!
	handlePlayerUpdate();
}

// Extract the key values from the HTML page
function getButtons()
{
	// id="playerkeylist" class="
	var titel = gStateDoc.match(/id="playerkeylist" class=\"(.+)\"/);		//	"
	return RegExp.$1;
}

function getCoverPng()
{
	var titel = gStateDoc.match(/src=\"(.+)\" id="coverart"/);		//	"
	return 'http://' + gHFURL + RegExp.$1;
}

function getCoverart()
{
	//	class="jazz"
	var titel = gStateDoc.match(/class=\"(.+)\"/);		//	"
	return RegExp.$1;
}

function getTimeline()
{
	//	style="width:30%"
	var titel = gStateDoc.match(/style=\"width:(.+%)\"/);	//	"
	return RegExp.$1;
}

function getTime()
{
	var titel = gStateDoc.match(/currtime\">(.+)<\/li>/);	//	"
	return RegExp.$1;
}

function getRemaining()
{
	var titel = gStateDoc.match(/currremain\">(.+)<\/li>/);	//	"
	return RegExp.$1;
}

function getTrack()
{
	var titel = gStateDoc.match(/currtrack\">(.+)<\/li>/);	//	"
	return RegExp.$1;
}

function getTitel()
{
	var titel = gStateDoc.match(/currtitle\">(.+)<\/p>/);	//	"
	var myMatch = RegExp.$1;
	return convert (myMatch);
}

function getArtist()
{
	gStateDoc.match(/currartist\">(.+)<\/p>/);	//	"
	var myMatch = RegExp.$1;
	return convert (myMatch);
}

function getAlbum()
{
	var titel = gStateDoc.match(/curralbum\">(.+)<\/p>/);	//	"
	var myMatch = RegExp.$1;
	return convert (myMatch);
}

function getComposer()
{
	var titel = gStateDoc.match(/currcomposer\">(.+)<\/p>/);	//	"
	var myMatch = RegExp.$1;
    if (myMatch == "&nbsp;")
        return "n/a";
	return convert (myMatch);
}

//
// We need to convert from html entities back to UTF-8 codes - this conversion
// handles a few characters only, hopefully this suffices
//
function convert(text)
{
	var Suche = /(&.+)/;
	var Ergebnis = Suche.test(text);
	if (Ergebnis == false)
		return text;
	
	var converted = text.replace(/&nbsp;/g, " ");
	converted = converted.replace(/&lt;/g, "<");
	converted = converted.replace(/&gt;/g, ">");
	converted = converted.replace(/&amp;/g, "&");
	converted = converted.replace(/&#38;/g, "&");
	converted = converted.replace(/&#39;/g, "'");
	converted = converted.replace(/&#(\d+);/g, String.fromCharCode(RegExp.$1));
	return converted;
}

// changeWorld() is called whenever a menu item is chosen in the widget's preferences.  It queries the
// menu to find out which option was chosen, applies the change to the widget, and saves the preference.

function changeWorld()
{	
	var world = document.getElementById("urlField");
	
	widget.setPreferenceForKey(world.value,"hifidelioURL");		// and save the new preference to disk

	gHFURL = world.value;			//	update the current values
	gStatusLink = 'http://' + gHFURL + '/cgi-bin/leonore.cgi?h=S';
} 


/*********************************/
// HIDING AND SHOWING PREFERENCES
/*********************************/

// showPrefs() is called when the preferences flipper is clicked upon.  It freezes the front of the widget,
// hides the front div, unhides the back div, and then flips the widget over.

function showPrefs()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	if (window.widget)
		widget.prepareForTransition("ToBack");		// freezes the widget so that you can change it without the user noticing
	
	front.style.display="none";		// hide the front
	back.style.display="block";		// show the back
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);		// and flip the widget over	

	document.getElementById('fliprollie').style.display = 'none';  // clean up the front side - hide the circle behind the info button
    document.getElementById('urlField').value = gHFURL;
}


// hidePrefs() is called by the done button on the back side of the widget.  It performs the opposite transition
// as showPrefs() does.

function hidePrefs()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	if (window.widget)
		widget.prepareForTransition("ToFront");		// freezes the widget and prepares it for the flip back to the front
	
	back.style.display="none";			// hide the back
	front.style.display="block";		// show the front
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);		// and flip the widget back to the front

    changeWorld();                      // save the URL
}





// PREFERENCE BUTTON ANIMATION (- the pref flipper fade in/out)

var flipShown = false;		// a flag used to signify if the flipper is currently shown or not.


// A structure that holds information that is needed for the animation to run.
var animation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0, firstElement:null, timer:null};


// mousemove() is the event handle assigned to the onmousemove property on the front div of the widget. 
// It is triggered whenever a mouse is moved within the bounds of your widget.  It prepares the
// preference flipper fade and then calls animate() to performs the animation.

function mousemove (event)
{
	if (!flipShown)			// if the preferences flipper is not already showing...
	{
		if (animation.timer != null)			// reset the animation timer value, in case a value was left behind
		{
			clearInterval (animation.timer);
			animation.timer  = null;
		}
		
		var starttime = (new Date).getTime() - 13; 		// set it back one frame
		
		animation.duration = 500;												// animation time, in ms
		animation.starttime = starttime;										// specify the start time
		animation.firstElement = document.getElementById ('flip');              // specify the element to fade
		animation.timer = setInterval ("animate();", 13);						// set the animation function
		animation.from = animation.now;											// beginning opacity (not ness. 0)
		animation.to = 1.0;														// final opacity
		animate();																// begin animation
		flipShown = true;														// mark the flipper as animated
	}
}

// mouseexit() is the opposite of mousemove() in that it preps the preferences flipper
// to disappear.  It adds the appropriate values to the animation data structure and sets the animation in motion.

function mouseexit (event)
{
	if (flipShown)
	{
		// fade in the flip widget
		if (animation.timer != null)
		{
			clearInterval (animation.timer);
			animation.timer  = null;
		}
		
		var starttime = (new Date).getTime() - 13;
		
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById ('flip');
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 0.0;
		animate();
		flipShown = false;
	}
}


// animate() performs the fade animation for the preferences flipper. It uses the opacity CSS property to simulate a fade.

function animate()
{
	var T;
	var ease;
	var time = (new Date).getTime();
		
	
	T = limit_3(time-animation.starttime, 0, animation.duration);
	
	if (T >= animation.duration)
	{
		clearInterval (animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	}
	else
	{
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		animation.now = computeNextFloat (animation.from, animation.to, ease);
	}
	
	animation.firstElement.style.opacity = animation.now;
}


// these functions are utilities used by animate()

function limit_3 (a, b, c)
{
    return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease)
{
    return from + (to - from) * ease;
}

// these functions are called when the info button itself receives onmouseover and onmouseout events

function enterflip(event)
{
	document.getElementById('fliprollie').style.display = 'block';
}

function exitflip(event)
{
	document.getElementById('fliprollie').style.display = 'none';
}

